 	  CPU	Z80
          org	0b800h 			; b800h Entrypoint on MZ7xx 
	  jp	start			; jump around definitions
	  jp	reset			; execution with printer reset
; written by KarlHeinz.Mau@t-online.de 1996, last modified may 12th, 1998
;
; This program prints on execution one character on a centronics printer
; connected to SHARP MZ7xx computers. The character to print is translated
; from Sharp's character code into ascii code.
;
; The printer used for printing at my site is NEC Pinwriter P6 plus. I can't 
; warrant the correct function with other printers because of the timing of
; the handshaking between my program and your printer. Test it and post me
; your practical experiences, please.
;
; Technical connection:
; You have to connect pin 1 (RDP) of the printer connector of the MZ to
; pin 1 (strobe) to your printer, MZ-pin 19 (IRT) with pin 16 (reset), MZ-pin
; 21 (RDA) with pin 11 (busy), MZ-pin 23 (STA) with pin 10 (ACK) and data
; RD1 to RD8 at MZ-pins 3, 5, 7, 9, 11, 13, 15, 17 to pins 2, 3, 4, 5, 6, 7,
; 8 and 9 and last MZ-pin 25 (GND) to pins 19 to 29.
;
; Program usage: 
; You can call this program in programs written by yourself. In this case
; you have to do the following:
;
;	  call	0b800h			; execute this program
;
; Your first execution of this program should be:
;
;	  call	0b803h			; exec with printer reset
;
; This resets the printer and the bus.
;
; My program sets the zero flag on return to your program. If the zero flag
; is set, the user has pressed the SHIFT- and BRK-keyes to stop the output.
; In this case you can interrupt your own program or do any other functions.
; For example:
;
;	  call	0b800h			; exec hardcopy
;	  jp	z,break			; if brk key go to label break
;	  ...	...			; if not ....
;


;	constants

porta:	  equ	0e000h			; 8255 Port A (write keyboard data)
portb:	  equ	0e001h			; 8255 Port B (read keyboard data)

;	printer translation table 

prttrtab: db	00h,01h,02h,0f0h,0f0h,0f0h,0f0h,07h
a08:	  db	08h,09h,0ah,0bh,0ch,0dh,0eh,0fh
a10:	  db	10h,11h,12h,13h,14h,15h,16h,17h
a18:	  db	18h,19h,1ah,1bh,1ch,1dh,1eh,1fh
a20:	  db	20h,21h,22h,23h,24h,25h,26h,27h
a28:	  db	28h,29h,2ah,2bh,2ch,2dh,2eh,2fh
a30:	  db	30h,31h,32h,33h,34h,35h,36h,37h
a38:	  db	38h,39h,3ah,3bh,3ch,3dh,3eh,3fh
a40:	  db	40h,41h,42h,43h,44h,45h,46h,47h
a48:	  db	48h,49h,4ah,4bh,4ch,4dh,4eh,4fh
a50:	  db	50h,51h,52h,53h,54h,55h,56h,57h
a58:	  db	58h,59h,5ah,5bh,5ch,5dh,0f0h,0f0h
a60:	  db	16 dup 0f0h
a70:	  db	11 dup 0f0h,7bh,4 dup 0f0h
a80:	  db	7dh,10 dup 0f0h,5eh,4 dup 0f0h
a90:	  db	5fh,0f0h,65h,60h,0f7h,0f0h,74h,67h
a98:	  db	68h,0f0h,62h,78h,64h,72h,70h,63h
aa0:	  db	71h,61h,7ah,77h,73h,75h,69h,0f0h
aa8:	  db	99h,6bh,66h,76h,0f0h,81h,0e1h,6ah
ab0:	  db	6eh,0f0h,9ah,6dh,3 dup 0f0h,6fh
ab8:	  db	6ch,8eh,94h,84h,0f0h,79h,7bh,0f0h
ac0:	  db	0b3h,15 dup 0f0h
ad0:	  db	16 dup 0f0h
ae0:	  db	0f0h,06h,14 dup 0f0h
af0:	  db	3 dup 0f0h,03h,4 dup 0f0h
af8:	  db	05h,0f0h,04h,9ch,3 dup 0f0h,0e3h

;  entry point on first execution resets printer

reset:	  ld	c,a			; save char to print
reset10:  IN	A,(0FEh)
	  BIT	0,A			; busy?
	  JR	Z,printch		; not busy
	  OR	40h			; set IRT
	  OUT	(0FEh),A		; reset printer
	  CALL	brktst			; test break key
	  JR	NZ,reset10		; no, loop
	  ret				; yes, goback

;  entry point on further executions

start:	  ld	c,a			; save char to print
start10:  IN	A,(0FEh)		; get state of printer
	  BIT	0,A			; busy?
	  JR	Z,printch		; no, continue
	  CALL	brktst			; yes, busy. brk key?
	  JR	NZ,start10		; no. Wait on ready
	  Ret				; goback with brk condition

;	go print 1 char

printch:  ld	a,c			; restore char to print
	  call	convprt			; convert to ascii
	  OUT	(0FFh),A		; data out
	  IN	A,(0FEh)		; get state of printer
	  AND	7Fh			; set strobe
	  OUT	(0FEh),A		; put strobe
	  IN	A,(0FEh)		; get state again
	  OR	80h			; reset strobe
	  OUT	(0FEh),A		; put out
prtch10:  IN	A,(0FEh)		; get state
	  BIT	0,A			; test busy
	  jr	Z,prtch20		; not busy, goback to caller
	  CALL	brktst			; brk key?
	  JR	NZ,prtch10		; no, wait on ready (printed)
	  Ret				; yes, goback, brk

prtch20:  cpl				; ffh/feh
	  and	a			; indicate no break (NZ)
	  ret

;	MZ-code to ascii conversion for centronics printer
;	in  accu=MZ-code
;	out accu=ascii
;	protected registers none

convprt:  LD	HL,prttrtab		; table display to ascii (printer)
	  LD	C,A
	  LD	B,00h			; bc is displacement into table
	  ADD	HL,BC			; address to char
	  LD	A,(HL)			; get translation char
	  CP	0F0h			; display code for space?
	  JR	NZ,conprt10		; no, continue
	  AND	20h			; yes, translate to 20h
conprt10: CP	0F0h			; display code f0h? (space)
	  RET	NZ			; no
	  AND	20h			; yes, convert to space
	  RET				; goback

;  test on brk key

brktst:	  CALL	brktst20		; test brk key
	  RET	NZ			; no, continue
brktst10: CALL	brktst20		; test again and
	  JR	Z,brktst10		; wait until unpressed
	  xor	a			; zero flag indicates break
	  ret

;	test shift-break

brktst20: LD	A,0F8h			; column brk/ctrl/shift
	  LD	(porta),A		; Port A 8255 send keyboard f8h
	  NOP				; wait 1 nop
	  LD	A,(portb)		; Port B 8255 receive keyboard data
	  CP	7Eh			; shift + brk ?
	  RET				; goback


